/*
 * clock.c
 *
 * Created: 2015-04-09 14:15:08
 *  Author: dsosnowski
 */ 

#define F_CPU 8000000UL
#include <avr/io.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include <util/delay.h>
#include <avr/wdt.h>
#include <avr/sleep.h>
#include <avr/pgmspace.h>
#include "defs.h"
//#include "ui.h"
#include "serial.h"
#include "clock.h"

struct {
	volatile uint32_t digital;
	volatile char tick;
	char area;
	char daylighten;
	char daylight;
} time;

struct {
	char seconds;
	char minutes;
	char hours;
	char dayofmonth;
	char month;
	char year;
	char dayofweek;
	char leap;		
} clock;

//0 - ND; 1 - PN; 2 - WT; 3 - SR; 4 - CZ; 5 - PT; 6 - SB
char days_const[12] = {31,28,31,30,31,30,31,31,30,31,30,31};

#define WEEKDAY_NUMBER		7
#define LEAPYEAR_OFFSET		1				//2000 = leapyear
#define LEAPYEAR_PERIOD		4
#define DAYS_ON_YEAR		365

#define SEC_ON_MIN			60
#define SEC_ON_HOUR			3600
#define SEC_ON_DAY			86400
#define SEC_ON_YEAR			31536000
#define SEC_ON_LYEAR		31622400
#define SEC_ON_FOUR			126230400		//366 + 365 + 365 + 365

#define WEEKDAY_OFFSET		6				//at 1 yan 2000 = SB


//------------------------------------------------------
void TimeInit(void)
{
	//ASSR = (1<<AS2);
	//TCCR2 = (1<<CS22 | 1<<CS20);
	//TIMSK |= (1<<TOIE2);
	
	MCUCR = (1<<ISC01);
	GICR = (1<<INT0);	
	sei();
	
	//time.area = 1;
	//time.daylighten = 1;
}

//-------------------------------------
ISR(INT0_vect)
{
	time.tick = 1;
	time.digital++;
}

//-------------------------------------
ISR(TIMER2_OVF_vect)
{
	time.tick = 1;
	time.digital++;
}

//------------------------------------------------------
char *TimeConfigGet(void)
{
	return &time.area;
}

//------------------------------------------------------
char TimeTick(void)
{
	char temp = time.tick;
	time.tick = 0;
	return temp;
}

//------------------------------------------------------
uint32_t *TimeGet(void)
{
	return &time.digital;
}

//------------------------------------------------------
uint32_t TimeVal(void)
{
	return time.digital;
}

//------------------------------------------------------
uint32_t TimeCorrectGet(uint32_t dtime)
{
	uint32_t temptime;
	
	if (time.area > 14)
		 temptime = (dtime - ((29 - time.area) * 3600));
	else
		 temptime = (dtime + (time.area * 3600));
		
	return temptime;
}

//------------------------------------------------------
char *ClockGet(void)
{
	return &clock.seconds;
}

//------------------------------------------------------
void ClockCalc(uint32_t dtime)
{
	uint32_t i;
	uint32_t temptime;

	//---day of week-----------------------------------
	i = (dtime / SEC_ON_DAY);
	//i += 1;
	i += WEEKDAY_OFFSET;	
	clock.dayofweek = (char)(i % WEEKDAY_NUMBER);
	
	//---year------------------------------------------
	i = (char)(dtime / SEC_ON_FOUR);		
	temptime = (dtime % SEC_ON_FOUR);
	clock.year = (i * 4);
	
	if (temptime < SEC_ON_LYEAR) 
	{
		clock.leap = 1;
	}
	else
	{
		clock.leap = 0;
		clock.year += 1;
		temptime -= SEC_ON_LYEAR;
		
		i = (temptime / SEC_ON_YEAR);
		clock.year += i;
		//clock.year += 1;
		temptime %= SEC_ON_YEAR;
	}
	
	//---month------------------------------------------
	if (clock.leap)
		days_const[1] = 29;
	else
		days_const[1] = 28;
	
	for(i = 0; i < 12; i++)
	{		
		if (temptime >= (days_const[i] * SEC_ON_DAY))
			temptime -= (days_const[i] * SEC_ON_DAY);
		else
		{
			clock.month = i;
			i = 12;	
		}
	}
	
	//---day of month------------------------------------
	clock.dayofmonth = (char)(temptime / SEC_ON_DAY);	
	temptime %= SEC_ON_DAY;
	
	//---hour--------------------------------------------
	clock.hours = (char)(temptime / SEC_ON_HOUR);
	temptime %= SEC_ON_HOUR;
	
	//---minutes-----------------------------------------
	clock.minutes = (char)(temptime / SEC_ON_MIN);
	temptime %= SEC_ON_MIN;

	//---sec---------------------------------------------
	clock.seconds = (char)(temptime);
}

//------------------------------------------------------
uint32_t TimeCalc(char *clock)
{
	char leap;
	uint32_t i;
	uint32_t temptime;
	
	//---year------------------------------------------
	i = (*(clock + 5) / 4);
	temptime = (i * SEC_ON_FOUR);

	i = (*(clock + 5) % 4);
	if (i == 0) 
	{
		leap = 1;
	}
	else
	{
		leap = 0;
		temptime += SEC_ON_LYEAR;
		i--;
	}
	temptime += (i * SEC_ON_YEAR);
	
	//---month------------------------------------------
	if (leap)
		days_const[1] = 29;
	else
		days_const[1] = 28;
	
	for(leap = 0; leap < *(clock + 4); leap++)
	{
		i = days_const[leap];
		temptime += (i * SEC_ON_DAY);
	}
	
	//---month------------------------------------------
	i = (uint32_t)(*(clock + 3));
	temptime += (i * SEC_ON_DAY);

	//---hour--------------------------------------------
	i = (uint32_t)(*(clock + 2));
	temptime += (i * SEC_ON_HOUR);
	
	//---minutes-----------------------------------------
	i = (uint32_t)(*(clock + 1));
	temptime += (i * SEC_ON_MIN);	

	//---sec---------------------------------------------
	i = (uint32_t)(*(clock));
	temptime += i;
	
	return temptime;
	//time.digital = temptime;
}

//------------------------------------------------------
char ClockDaylight(void)
{
	if (time.daylighten)	
	{
		//---daylight time check---------------------------
		if ((clock.month > 2) && (clock.month < 9))
			time.daylight = 1;
		else if ((clock.month < 2) || (clock.month > 9))
			time.daylight = 0;
	
		else if (clock.month == 2)	//only MARCH
		{
			if (clock.dayofmonth < 24)
				time.daylight = 0;
			else
			{
				if (clock.dayofweek <= (clock.dayofmonth - 24))
				{
					if (clock.dayofweek > 0)
						time.daylight = 1;
					else
					{
						if ((clock.hours >= 1) && (clock.minutes >= 0) && (clock.seconds >= 0))
							time.daylight = 1;
						else
							time.daylight = 0;
					}
				}
				else
				time.daylight = 0;
			}
		}
	
		else if (clock.month == 9)	//only SEPT
		{
			if (clock.dayofmonth < 24)
			time.daylight = 1;
			else
			{
				if (clock.dayofweek <= (clock.dayofmonth - 24))
				{
					if (clock.dayofweek > 0)
					time.daylight = 0;
					else
					{
						if ((clock.hours >= 1) && (clock.minutes >= 0) && (clock.seconds >= 0))
							time.daylight = 0;
						else
							time.daylight = 1;
					}
				}
				else
				time.daylight = 1;
			}
		}
	}
	else time.daylight = 0;
		
	return time.daylight;
}
