/*
 * dstemp.c
 *
 * Created: 2012-03-15 08:12:33
 *  Author: dsosnowski
 */ 

#define F_CPU 8000000
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include "defs.h"
#include "dstemp.h"


#define ONEW_L				(SW_PORT &= ~(1<<TERM_PIN))
#define ONEW_H				(SW_PORT |=  (1<<TERM_PIN))
#define ONEW_SETO			(SW_DDR |=  (1<<TERM_PIN))
#define ONEW_SETI			(SW_DDR &= ~(1<<TERM_PIN))
#define ONEW_GET			(SW_PIN &   (1<<TERM_PIN))


//ds.step
#define DS_IDLE				1
#define DS_CONVERTING		2

//ds.status
#define DS_BUSY				0
#define DS_OK				1
#define DS_ERR				2
#define DS_NO_CONNECT		3


#define DS_READ_ROM			0x33
#define DS_SKIP_ROM			0xCC
#define DS_START_CONV		0x44
#define DS_RAED_RESULT		0xBE

#define DS_TYPE_NOREAD		0
#define	DS_TYPE_18B20		0x28
#define	DS_TYPE_18S20		0x10


struct {
	char step;
	char channel;
	char type;
	char status;
	char ecounter;
	int16_t data;
	int16_t temperature;
	char view;
} ds;


//-------------------------------------------------------
//simple delay
void SystemUsDelay(char us)
{
	TCCR0 = 0;
	//SFIOR = (1<<PSR10);
	TIFR |= (1<<TOV0);
	
	us -= 1;
	TCNT0 = (0xFF - us);
	TCCR0 = (1<<CS01);
	
	while((TIFR & (1<<TOV0)) == 0);
}

//-------------------------------------------------------
int16_t DSTempGet(void)
{
	return ds.temperature;
}

//-------------------------------------------------------
char DSStatusGet(void)
{
	if (ds.ecounter >= DS_ERR_MAX)
		return DS_NO_CONNECT;
	else
		return ds.status;
}

//--------------------------------
char *DSConfigGet(void)
{
	return &ds.view;
}

//-------------------------------------------------------
char DSProcessStep(char step)
{
	if (step > 0) ds.step = step;
	return ds.step;
}

//------------------------------------
//must be execute about 1/1s, return = 2 then result ready
char DSProcess(void)
{
	if (ds.step <= DS_IDLE)
	{
		ds.status = DS_BUSY;
		
		if (DSGetType() == DS_TYPE_NOREAD)
			ds.status = DS_ERR;
			
		if (DSReset() == DS_OK)
		{
			DSWriteByte(DS_SKIP_ROM);		//skip ROM
			DSWriteByte(DS_START_CONV);		//convert
			//power for convert
			ONEW_H;
			ONEW_SETO;
			//ds.status = DS_CONVERTING;
		}
		else
			ds.status = DS_ERR;
		
	}
	//else
	if (ds.step == DS_CONVERTING)
	{
		if (DSReset() == DS_OK)
		{
			DSWriteByte(DS_SKIP_ROM);		//skip ROM
			DSWriteByte(DS_RAED_RESULT);		//read
			ds.data = DSReadByte();
			ds.data |= ((DSReadByte())<<8);
			TempConvert();
		}
		else
			ds.status = DS_ERR;
			
		if (ds.status == DS_BUSY)
		{
			ds.ecounter = 0;
			ds.status = DS_OK;
		}
	}
	
	if (ds.status == DS_ERR)
	{
		if (ds.ecounter < DS_ERR_MAX) 
			ds.ecounter++;
	}
	
	return ds.step;
}

//-------------------------------------
char DSReset (void)
{
	char status;

	status = DS_ERR;

	ONEW_L;
	ONEW_SETO;
	//SystemUsDelay(800);
	SystemUsDelay(255);SystemUsDelay(255);SystemUsDelay(255);

		ONEW_SETI;
		ONEW_H;
		SystemUsDelay(100);

		if (ONEW_GET == 0)
		{
			//SystemUsDelay(600);
			SystemUsDelay(200);SystemUsDelay(200);SystemUsDelay(200);
			if (ONEW_GET != 0) status = DS_OK;
			//else - short to GND
		}
		//else - no DS	
	return status;
}

//-------------------------------------
char DSGetType (void)
{
	char i;

	ds.type = DS_TYPE_NOREAD;

	if (DSReset() == DS_OK)
	{
		DSWriteByte(DS_READ_ROM);				//read ROM
		i = DSReadByte();

		if ((i == DS_TYPE_18B20) || (i == DS_TYPE_18S20))
			ds.type = i;
	}
	
	return ds.type;
}

//-------------------------------------
void DSWriteByte(char byte)
{
	char i;
	for (i=0; i<8; i++)
	{
		DSWriteBit(byte & 0x01);
		byte >>= 1;
	}
}

//-------------------------------------
void DSWriteBit(char bit)
{
	ONEW_L;
	ONEW_SETO;
	SystemUsDelay(6);

	if (bit > 0) ONEW_H;

	SystemUsDelay(64);

	ONEW_H;
	SystemUsDelay(20);
}

//-------------------------------------
char DSReadByte(void)
{
	char i;
	char temp = 0;

	for (i=0; i<8; i++)
	{
		temp >>= 1;
		if (DSReadBit() > 0) temp |= 0x80;
	}
	return temp;
}

//-------------------------------------
char DSReadBit(void)
{
	char state = 0;

	ONEW_L;
	ONEW_SETO;
	SystemUsDelay(6);
	ONEW_SETI;
	ONEW_H;	
	
	SystemUsDelay(6);
	if (ONEW_GET != 0) state = 1;

	SystemUsDelay(78);
	return state;
}

//-------------------------------------
void TempConvert(void)
{
	int32_t buffer = 0;
	
		if (ds.type== DS_TYPE_18B20)
		{
			buffer = (int32_t)(ds.data);
			buffer *= 625;
			buffer /= 1000;
		}
		else if (ds.type == DS_TYPE_18S20)
		{
			buffer = (ds.data * 5);
		}
		else 
			ds.status = DS_ERR;

		if ((ds.status) == DS_BUSY)
		{
			if ((buffer <=  1250) && (buffer >= -550))
				ds.temperature = (int16_t)buffer;
			else
				ds.status = DS_ERR;
		}
}
