/*
 * rtc_i2c.c
 *
 * Created: 2015-04-09 14:16:08
 *  Author: dsosnowski
 */ 


#define F_CPU 8000000UL
#include <avr/io.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include <util/delay.h>
#include "defs.h"
#include "serial.h"
#include "rtc_i2c.h"

/*#define I2C_PORT		PORTC
#define I2C_DDR			DDRC
#define I2C_PIN			PINC
#define SDA				4
#define SCL				5
*/

//#define I2C_FREQ_VAL	126		//30kHz
#define I2C_FREQ_VAL	8		
//#define I2C_SLAVE_ADR			0xA0	//PCF8583
//#define I2C_SLAVE_WADR		0xA0	//PCF8583
//#define I2C_SLAVE_RADR		0xA1	//PCF8583
#define I2C_SLAVE_ADR			0xDE	//MCP7940
#define I2C_SLAVE_WADR			0xDE	//MCP7940
#define I2C_SLAVE_RADR			0xDF	//MCP7940

#define I2C_STATUS_START		0x08
#define I2C_STATUS_SLAR_ACK		0x40
#define I2C_STATUS_DATAR_ACK	0x50
#define I2C_STATUS_DATAR_NACK	0x58

#define I2C_STATUS_SLAW_ACK		0x18
#define I2C_STATUS_DATAW_ACK	0x28
#define I2C_STATUS_DATAW_NACK	0x30

#define I2C_NO_ACK				0
#define I2C_ACK					1
#define I2C_LAST_DATA			2

#define RTC_STATUS_IDLE			0
#define RTC_STATUS_OK			1
#define RTC_STATUS_I2C_ERR		2
#define RTC_STATUS_RTC_ERR		3
#define RTC_STATUS_TIME_ERR		4

#define RTC_START_ADR			0x00
#define RTC_BYTES_TO_READ		8

struct {
	char status;
	char buffer[8];
	char seconds;
	char minutes;
	char hours;
	char dayofmonth;
	char month;
	char year;
} rtc;


//----------------------------------------------
void RtcInit(void)
{
	char result;
	char i;
	
	//-----------------------------------------------
	i2c_init();
	_delay_ms(50);
	sei();
	
	//-----------------------------------------------
	i = 0;
	result = RTC_STATUS_IDLE;	
	while (result == RTC_STATUS_IDLE)
	{
		if (i2c_open(I2C_SLAVE_WADR) == RTC_STATUS_OK)
		{
			result = RTC_STATUS_OK;
		}
		else
		{
			i2c_stop();			
			_delay_ms(50);
		}
		i++;
		if (i > 5) result = RTC_STATUS_RTC_ERR;
	}
	i2c_stop();
	_delay_ms(50);	
	rtc.status = result;
	
	SerialMsg("i2c init ");
	SerialSimpleVal(rtc.status);
}

//----------------------------------------------
void RtcConfig(void)
{
	char reconfig = 0;
	char result;
	char i;
	Rtc2Buff();
		
	if (rtc.status == RTC_STATUS_OK)
	{
		if ((rtc.buffer[0] & (1<<7)) == 0)
		{
			rtc.buffer[0] |= (1<<7);
			reconfig = 1;
		}
		
		if ((rtc.buffer[2] & (1<<6))	> 0)
		{
			rtc.buffer[2] &= ~(1<<6);
			reconfig = 1;
		}
			
		if (rtc.buffer[7] != (1<<6))
		{
			rtc.buffer[7] = (1<<6);
			reconfig = 1;
		}
		
		//-----------------------------------------------
		if (reconfig)
		{	
			result = i2c_open(I2C_SLAVE_WADR);
			if (result == RTC_STATUS_OK)
			{
				i2c_write(RTC_START_ADR);

				i = 0;
				while (i<8)
				{
					i2c_write(rtc.buffer[i]);
					i++;
				}
			}
			i2c_stop();
			_delay_ms(50);
	
			rtc.status = result;		
		}
	}
	
	SerialMsg("i2c conf ");
	SerialSimpleVal(rtc.status);
}

//----------------------------------------------
void RtcStop(void)
{
	TCCR2 = 0;
	TIMSK &= ~(1<<TOIE2);
}

//----------------------------------------------
void RtcDisable(void)
{
	TIMSK &= ~(1<<TOIE2);
}

//----------------------------------------------
void RtcEnable(void)
{
	TIMSK |= (1<<TOIE2);
}

//-------------------------------------------
char *RtcRead(void)
{
	char i = 0;

	Rtc2Buff();
	
	if (rtc.status == RTC_STATUS_OK)
	{		
		i = (rtc.buffer[0] & 0x70);
		i = ((i >> 4) * 10);
		i = ( i + (rtc.buffer[0] & 0x0F));
		//*dest = i; dest++;
		rtc.seconds = i;

		i = (rtc.buffer[1] & 0x70);
		i = ((i >> 4) * 10);
		i = ( i + (rtc.buffer[1] & 0x0F));
		//*dest = i; dest++;
		rtc.minutes = i;

		i = (rtc.buffer[2] & 0x30);
		i = ((i >> 4) * 10);
		i = ( i + (rtc.buffer[2] & 0x0F));
		//*dest = i; dest++;
		rtc.hours = i;
		
		i = (rtc.buffer[4] & 0x30);
		i = ((i >> 4) * 10);
		i = ( i + (rtc.buffer[4] & 0x0F));
		i -= 1;
		//*dest = i; dest++;
		rtc.dayofmonth = i;
		
		i = (rtc.buffer[5] & 0x10);
		i = ((i >> 4) * 10);
		i = ( i + (rtc.buffer[5] & 0x0F));
		i -= 1;
		//*dest = i; dest++;
		rtc.month = i;

		i = (rtc.buffer[6] & 0xF0);
		i = ((i >> 4) * 10);
		i = ( i + (rtc.buffer[6] & 0x0F));
		//*dest = i;
		rtc.year = i;
	}
	
	SerialMsg("i2c read ");
	SerialSimpleVal(rtc.status);
	
	return &rtc.seconds;	
}


//-------------------------------------------
void RtcSet(char *src)
{
	char result;
	char i;
	
	//if (rtc.status == RTC_STATUS_OK)
	//{
		i = (*src / 10);
		i = (i<<4);
		i = (i + (*src % 10));
		rtc.buffer[0] = (i | (1<<7));
		src++;

		i = (*src / 10);
		i = (i<<4);
		i = (i + (*src % 10));
		rtc.buffer[1] = i;
		src++;

		i = (*src / 10);
		i = (i<<4);
		i = (i + (*src % 10));
		rtc.buffer[2] = i;
		src++;

		i = (*src / 10);
		i = (i<<4);
		i = (i + (*src % 10));
		rtc.buffer[4] = (i + 1);
		src++;

		i = (*src / 10);
		i = (i<<4);
		i = (i + (*src % 10));
		rtc.buffer[5] = (i + 1);
		src++;

		i = (*src / 10);
		i = (i<<4);
		i = (i + (*src % 10));
		rtc.buffer[6] = i;
		
		rtc.buffer[7] = (1<<6);		
		
		result = i2c_open(I2C_SLAVE_WADR);
		if (result == RTC_STATUS_OK)
		{
			i2c_write(RTC_START_ADR);

			i = 0;
			while (i<RTC_BYTES_TO_READ)
			{
				i2c_write(rtc.buffer[i]);
				i++;
			}
		}
		i2c_stop();
		rtc.status = result;
		
		SerialMsg("i2c set ");
		SerialSimpleVal(rtc.status);

	//}
}

//-----------------------------------------
void Rtc2Buff(void)
{
	char result = 0;
	char i = 0;

	//---clear buff-------------------------
	while (i < 8)
	{
		rtc.buffer[i] = 0;
		i++;
	}

	//---------------------------------------
	result = i2c_open(I2C_SLAVE_WADR);	
	if (result == RTC_STATUS_OK)
		result = i2c_write(RTC_START_ADR);
	i2c_stop();
	
	if (result != RTC_STATUS_OK) result = RTC_STATUS_RTC_ERR;
	
	if (result == RTC_STATUS_OK)
	{
		result = i2c_open(I2C_SLAVE_RADR);	
		if (result == RTC_STATUS_OK)
		{
			i = 0;
			while (i < RTC_BYTES_TO_READ)
			{
				i2c_read(&rtc.buffer[i], I2C_ACK);
				i++;
			}
			i2c_read(&rtc.buffer[i], I2C_NO_ACK);
		}
		i2c_stop();
	}
	
	rtc.status = result;
}


//################### I2C part ################################
//-----------------------------------------
void i2c_init(void)
{
	TWBR = I2C_FREQ_VAL;			//divider
	TWSR = 0x00;					//presc = 0;
	TWCR |= (1<<TWEN);				//enable TWI
}

//-----------------------------------------
char i2c_open(char address)
{
	i2c_stop();
	_delay_ms(5);

	TWCR = (1<<TWINT | 1<<TWSTA | 1<<TWEN);	//send start

	while (!(TWCR & (1<<TWINT)));	//wait for int
	if (TWSR != I2C_STATUS_START) 	//check status
	return RTC_STATUS_I2C_ERR;		//if not start then error

	TWDR = address;					//prepare address
	TWCR = (1<<TWINT | 1<<TWEN);	//enable send address

	while (!(TWCR & (1<<TWINT)));	//wait for int

	//if no ack then error
	if(!((TWSR == I2C_STATUS_SLAR_ACK) || (TWSR == I2C_STATUS_SLAW_ACK)))
	return RTC_STATUS_RTC_ERR;

	return RTC_STATUS_OK;			//finished
}

//-----------------------------------------
char i2c_read(char* dest_pt, char ack)
{
	if (ack)
	TWCR = (1<<TWINT | 1<<TWEN | 1<<TWEA);	//clear int, ack
	else
	TWCR = (1<<TWINT | 1<<TWEN);			//clear int, not ack

	while (!(TWCR & (1<<TWINT)));			//wait for int
	if (TWSR == I2C_STATUS_DATAR_ACK)	 	//check status
	{
		*dest_pt = TWDR;
		return RTC_STATUS_OK;					//finished
	}
	else if (TWSR == I2C_STATUS_DATAR_NACK)
	{
		*dest_pt = TWDR;
		return I2C_LAST_DATA;
	}
	else
	return I2C_STATUS_DATAR_ACK;
}

//-----------------------------------------
char i2c_write(char data)
{
	TWDR = data;
	TWCR = (1<<TWINT | 1<<TWEN);	//enable send address

	while (!(TWCR & (1<<TWINT)));			//wait for int
	if (TWSR == I2C_STATUS_DATAW_ACK)	 	//check status
	return RTC_STATUS_OK;					//finished
	else if (TWSR == I2C_STATUS_DATAW_NACK)
	return I2C_LAST_DATA;
	else
	return I2C_STATUS_DATAW_ACK;
}

//-----------------------------------------
void i2c_stop(void)
{
	TWCR = (1<<TWINT | 1<<TWEN | 1<<TWSTO);
}
