/*
 * settings.c
 *
 * Created: 2015-04-15 11:54:50
 *  Author: dsosnowski
 */ 

#define F_CPU 8000000UL
#include <avr/io.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include <util/delay.h>
#include <avr/wdt.h>
#include <avr/sleep.h>
#include <avr/pgmspace.h>
//#include "ee_mem.h"
#include "rtc_i2c.h"
#include "defs.h"
#include "ui.h"
#include "clock.h"
#include "settings.h"
#include "serial.h"
#include "ee_mem.h"
#include "dstemp.h"


struct {
	char seconds;
	char minutes;
	char hours;
	char dayofmonth;
	char month;
	char year;
}sett;

//--------------------------------
void SettInit(char *src)
{
	char i;
	char *dest;
	
	dest = &sett.seconds;
	//src = ClockGet();
	
	for (i = 0; i < 6; i++)
	{
		*dest = *src;
		dest++;
		src++;
	}
}

//--------------------------------
char *SettGet(void)
{
	return &sett.seconds;
}

//--------------------------------
void SettSave(void)
{
	EEwrite(6, *TimeConfigGet());
	EEwrite(7, *(TimeConfigGet()+1));
	EEwrite(8, *SerialConfigGet());
	EEwrite(9, *DSConfigGet());
}

//--------------------------------
void SettLoad(void)
{
	char val;
	
	val = EEread(6);
	if (val > 28) val = 1;
	*TimeConfigGet() = val;		//area
	
	val = EEread(7);
	if (val > 1) val = 1;
	*(TimeConfigGet()+1) = val;	//daylight
	
	val = EEread(8);
	if (val > 3) val = 3;
	*SerialConfigGet() = val;	//baud
	
	val = EEread(9);
	if (val > 1) val = 1;
	*DSConfigGet() = val;		//temp
}

//--------------------------------
void SettChange(char level, char param, char action)
{
	char *dest;
	
	if (level == 1)
	{
		if (param == 1)
		{
			if (action == UI_SWITCH2)
			{
				sett.dayofmonth += 1;
				if (sett.dayofmonth >= 31) sett.dayofmonth = 0;
			}
			if (action == UI_SWITCH1)
			{
				sett.dayofmonth -= 1;
				if (sett.dayofmonth >= 31) sett.dayofmonth = 30;
			}
		}
		if (param == 2)
		{
			if (action == UI_SWITCH2)
			{
				sett.month += 1;
				if (sett.month >= 12) sett.month = 0;
			}
			if (action == UI_SWITCH1)
			{
				sett.month -= 1;
				if (sett.month >= 12) sett.month = 11;
			}
		}
		if (param == 3)
		{
			if (action == UI_SWITCH2)
			{
				sett.year += 1;
				if (sett.year > 99) sett.year = 0;
			}
			if (action == UI_SWITCH1)
			{
				sett.year -= 1;
				if (sett.year > 99) sett.year = 99;
			}
		}
	}
	if (level == 2)
	{
		if (param == 1)
		{
			if (action == UI_SWITCH2)
			{
				sett.hours += 1;
				if (sett.hours >= 24) sett.hours = 0;
			}
			if (action == UI_SWITCH1)
			{
				sett.hours -= 1;
				if (sett.hours >= 24) sett.hours = 23;
			}
		}
		if (param == 2)
		{
			if (action == UI_SWITCH2)
			{
				sett.minutes += 1;
				if (sett.minutes >= 60) sett.minutes = 0;
			}
			if (action == UI_SWITCH1)
			{
				sett.minutes -= 1;
				if (sett.minutes >= 60) sett.minutes = 59;
			}
		}
		if (param == 3)
		{
			if (action == UI_SWITCH2)
			{
				sett.seconds += 1;
				if (sett.seconds >= 60) sett.seconds = 0;
			}
			if (action == UI_SWITCH1)
			{
				sett.seconds -= 1;
				if (sett.seconds >= 60) sett.seconds = 59;
			}
		}
	}

	if (level == 3)			//time zone
	{
		if (param == 1)
		{
			dest = TimeConfigGet();
		
			if (action == UI_SWITCH2)
			{
				(*dest)++;
				if (*dest >= 29) *dest = 0;
			}
			if (action == UI_SWITCH1)
			{
				(*dest)--;
				if (*dest >= 29) *dest = 28;
			}
		}
		
		if (param == 3)		//daylight
		{
			dest = (TimeConfigGet() + 1);
			
			if (action)
			{
				(*dest)++;
				if (*dest > 1) *dest = 0;
			}
		}
	}

	if (level == 4)			//config
	{
		if (param == 1)		//baud
		{
			dest = SerialConfigGet();
			
			if (action == UI_SWITCH2)
			{
				(*dest)++;
				if (*dest > 3) *dest = 0;
			}
			if (action == UI_SWITCH1)
			{
				(*dest)--;
				if (*dest > 3) *dest = 3;
			}
		}
		
		if (param == 3)		
		{
			dest = DSConfigGet();
			
			if (action)
			{
				(*dest)++;
				if (*dest > 1) *dest = 0;
			}
		}
	}
}