/*
 * the_clock.c
 *
 * Created: 2015-04-09 14:14:46
 *  Author: dsosnowski
 */ 

#define F_CPU 8000000UL
#include <avr/io.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include <util/delay.h>
#include <avr/wdt.h>
#include <avr/sleep.h>
#include <avr/pgmspace.h>
//#include "ee_mem.h"
#include "rtc_i2c.h"
#include "defs.h"
#include "ui.h"
#include "clock.h"
#include "settings.h"
#include "serial.h"
#include "wdog.h"
#include "dstemp.h"


void SystemHardInit(void);
void SystemKbdActions(char press);
void SystemDisplay(void);
void SystemBlink(void);
void SystemIntro(void);


#define SYSTEM_DOT_BLINK		22
#define SYSTEM_TIME_NOINIT		0
#define SYSTEM_TIME_INIT		1
#define SYSTEM_TIME_SYNCHRO		2
#define SYSTEM_TIME_SYNCHRO_OK	3

#define SYSTEM_SYNCHRO_PERIOD	30
#define SYSTEM_SYNCHRO_COND		10813		//3h 13s

#define SYSTEM_TEMP_PERIOD		71

#define SYSTEM_IDLE_VAL			15
#define SYSTEM_LEVEL_MAX		2
#define SYSTEM_PARAM_MAX		3

#define INTRO_DELAY				100


struct {	
	char status;
	char tick;
	uint16_t speriod;
	char tperiod;
	char refresh;
	char level;
	char param;
	char save;
	char blink;
	char idle;	
} system;


//-----------------------------------------------------------
int main(void)
{	
	SystemHardInit();	
	SettLoad();

	SerialInit();
	SerialMsg("THE CLOCK");
	SerialEnable(1);
	
	UiKbdInit();
	RtcInit();
	RtcConfig();
	
	TimeInit();
	UiDispInit();	
	WdtSet(7);
	
	system.status = SYSTEM_TIME_NOINIT;
	system.idle = SYSTEM_IDLE_VAL;
	system.tperiod = SYSTEM_TEMP_PERIOD;
	
	_delay_ms(200);
	if (UiKbdState() == UI_SWITCH1) system.level = 3;
	if (UiKbdState() == UI_SWITCH2) system.level = 4;
	
	SystemIntro();
	
    while(1)
    {
		//-----------------------------------------------------------
		if (UiKbdScan())
		{			
			system.idle = SYSTEM_IDLE_VAL;
			SystemKbdActions(UiKbdGet(0));
			system.refresh = 1;
			UiKbdGet(1);
		}

		//-----------------------------------------------------------
		if (TimeTick())
		{
			system.tick = 1;
			
			//-----------------------------------------------------------
			if (system.status == SYSTEM_TIME_NOINIT)
			{
				system.save = 3;
				system.status = SYSTEM_TIME_INIT;
			}

			//-----------------------------------------------------------
			if (system.save)
			{
				if (system.save == 1) RtcSet(SettGet());
				if (system.save == 2) RtcSet(SerialClockGet());	
				if (system.save == 3) SettSave();	
				*TimeGet() = TimeCalc(RtcRead());
				system.save = 0;
			}
			
			//ClockCalc(TimeVal());
			//ClockCalc(TimeCorrectCalc());
			system.refresh = 1;
			system.blink = 0;
		}
				
		//-----------------------------------------------------------
		if (system.status == SYSTEM_TIME_SYNCHRO_OK)
		{
			if (system.tick)
			{
				if ((TimeVal() % SYSTEM_SYNCHRO_COND) == 0)
				{
					system.status = SYSTEM_TIME_SYNCHRO;
					system.speriod = SYSTEM_SYNCHRO_PERIOD;
					SerialEnable(1); 
				}
			}
		}
		else
		//if (system.status <= SYSTEM_TIME_SYNCHRO)
		{
			if (system.tick)
			{
				if (system.speriod)
					system.speriod--;
				else
					system.status = SYSTEM_TIME_INIT;
			}
		}
		
		//-----------------------------------------------------------
		if (system.status <= SYSTEM_TIME_SYNCHRO)
		{
			if (SerialStatus() == SERIAL_READY)
			{
				if (SerialClock() == 1)
				{
					system.save = 2;
					system.status = SYSTEM_TIME_SYNCHRO_OK;
				}
				SerialEnable(1);
			}			
		}

		//-----------------------------------------------------------
		if (system.idle) 
		{
			if (system.tick)
			{
				system.idle--;
				if (system.idle == 0)
				{
					system.level = 0;
					system.param = 0;
					system.save = 0;
				}
			}
		}

		//-----------------------------------------------------------
		if (system.tick)
		{	
			if (DSProcessStep(0) <= 1)
			{
				DSProcessStep(1);
				DSProcess();
				DSProcessStep(2);
			}
			else
			{
				DSProcess();
				DSProcessStep(1);
			}
			
			if (*DSConfigGet())
			{
				if (system.level == 0)
				{
					if (system.tperiod)
						system.tperiod--;
					else
					{
						if (DSStatusGet() <= DS_OK)
						system.param = 2;
						system.idle = (SYSTEM_IDLE_VAL / 3);
						system.tperiod = SYSTEM_TEMP_PERIOD;
					}
				}
			}
		}
		
		//-----------------------------------------------------------
		SystemDisplay();

		SystemBlink();
		
		UIBrightness();
				
		//-----------------------------------------------------------
		system.tick = 0;
		_delay_ms(30);
		WdtReset();
    }
}

//-----------------------------------------------------------
void SystemHardInit(void)
{
	SW_PORT = 0x0E;
	SW_DDR = 0x01;
	
	ACSR = (1<<ACD);	//turn off comparator
}

//-----------------------------------------------------------
void SystemKbdActions(char press)
{
			//---enter menu---------------------
			if (UiKbdGet(0) == (UI_SWITCH1 | UI_SWITCH_HOLD))
			{
				if (system.level == 0)
				{
					system.save = 0;
					system.param = 1;
					system.level = 1;
					SettInit(ClockGet());				
				}
				else
				{
					system.param++;
					if (system.level == 2)		//settings time
					{
						if (system.param > 3)	//next step is exit
						{
							system.level = 0;
							system.param = 0;
						}
					}
					if (system.level == 1)		//settings date
					{
						if (system.param > 3)	//next step is settings time 
						{
							system.param = 1;
							system.level = 2;						
						}
					}
					if (system.level >= 3)		//settings time zone or settings
					{
						if (system.param > 3)	//next step is settings time
						{
							system.level = 0;
							system.param = 0;
						}
						if (system.param == 2)	//next step is settings time
						{
							system.param = 3;
						}
					}
				}
			}
			//---manual synchro---------------------
			if (UiKbdGet(0) == (UI_SWITCH2 | UI_SWITCH_HOLD))
			{
				if (system.level == 0)
				{
					if (system.status > SYSTEM_TIME_INIT)
					{
						SerialEnable(1); 
						system.status = SYSTEM_TIME_INIT;
					}
				}
				else
				{
					system.param++;
					if (system.level == 2)		//settings time
					{
						if (system.param > 3)	//next step is exit
						{
							system.level = 0;
							system.param = 0;
						}
					}
					if (system.level == 1)		//settings date
					{
						if (system.param > 3)	//next step is settings time
						{
							system.param = 1;
							system.level = 2;
						}
					}
					if (system.level >= 3)		//settings time zone or settings
					{
						if (system.param > 3)	//next step is settings time
						{
							system.level = 0;
							system.param = 0;
						}
						if (system.param == 2)	//next step is settings time
						{
							system.param = 3;
						}
					}
				}
			}	
					
			if (UiKbdGet(0) == UI_SWITCH1)
			{
				//---disp date---------------------
				if (system.level == 0)
				{
					if (system.param)
						system.param = 0;
					else
						system.param = 1;
					system.idle = (SYSTEM_IDLE_VAL / 3);
				}
				//---next menu---------------------
				else	//if (system.level > 0)					
				{
					if (system.param)
					{
						if (system.level <= 2)
							system.save = 1;		//save time
						else	
							system.save = 3;		//save sett	
						SettChange(system.level, system.param, UI_SWITCH1);						
					}
				}					
			}
			
			if (UiKbdGet(0) == UI_SWITCH2)
			{
				//---disp temp---------------------
				if (system.level == 0)
				{
					if (system.param) 
						system.param = 0;
					else
					{
						if (DSStatusGet() <= DS_OK) 
							system.param = 2;
						system.idle = (SYSTEM_IDLE_VAL / 3);
						system.tperiod = SYSTEM_TEMP_PERIOD;
					}
				}
				//---next param---------------------
				else//if (system.level > 0)
				{
					if (system.param)
					{
						if (system.level <= 2)
							system.save = 1;		//save time
						else
							system.save = 3;		//save sett
						SettChange(system.level, system.param, UI_SWITCH2);
					}					
				}
			}
}

//-----------------------------------------------------------
void SystemDisplay(void)
{
	uint32_t val;
		//-----------------------------------------------------------
		if (system.refresh)
		{
			system.refresh = 0;
			
			if (system.level == 0)
			//val = TimeCalc(ClockGet());
			val = TimeVal();
			else
			{
				if (system.param == 0)
				val = TimeVal();
				else
				val = TimeCalc(SettGet());
			}
			
			val = TimeCorrectGet(val);
			ClockCalc(val);
			if (ClockDaylight() == 1)
			val += 3600;
			ClockCalc(val);
			
			if (system.level == 0)
			{
				if (system.param == 0) UiDispClock(ClockGet());
				if (system.param == 1) UiDispDate(ClockGet() + 3);
				if (system.param == 2) UiDispTemp(DSTempGet());
			}
			if (system.level == 1)
			{
				UiDispDate(ClockGet() + 3);
			}
			if (system.level == 2)
			{
				UiDispClock(ClockGet());				
			}
			if (system.level == 3)
			{
				UiDispConfigTimeZone(TimeConfigGet());
			}
			if (system.level == 4)
			{
				UiDispConfigGps(SerialConfigGet());
				UiDispConfigTemp(DSConfigGet());		
			}
		}
}

//-----------------------------------------------------------
void SystemBlink(void)
{
		//-----------------------------------------------------------
		if (system.blink < SYSTEM_DOT_BLINK)
		{
			system.blink++;
			
			if (system.level == 0)
			{
				if (system.status <= SYSTEM_TIME_SYNCHRO)
				{
					if (system.blink >= (SYSTEM_DOT_BLINK / 4))
					UiDispMask(4);
				}
				else
				{
					if (system.blink >= SYSTEM_DOT_BLINK)
					UiDispMask(4);
				}
			}
			else
			{
				if (system.blink >= SYSTEM_DOT_BLINK)
				UiDispMask(system.param);
			}
		}
}

//-----------------------------------------------------------
void SystemIntro(void)
{
	UiDispDirec(0,0,0,0,0,0);
	_delay_ms(INTRO_DELAY);
	UiDispDirec(0,183,0,0,0,0);
	_delay_ms(INTRO_DELAY);
	UiDispDirec(0,183,61,0,0,0);
	_delay_ms(INTRO_DELAY);
	UiDispDirec(0,183,61,50,0,0);
	_delay_ms(500);
	WdtReset();
	
	UiDispDirec(0,0,0,0,0,0);
	_delay_ms(INTRO_DELAY);
	UiDispDirec(174,0,0,0,0,0);
	_delay_ms(INTRO_DELAY);
	UiDispDirec(174,174,0,0,0,0);
	_delay_ms(INTRO_DELAY);
	UiDispDirec(174,174,155,0,0,0);
	_delay_ms(INTRO_DELAY);	
	UiDispDirec(174,174,155,155,0,0);
	_delay_ms(700);
	WdtReset();
}