/*
 * ui.c
 *
 * Created: 2015-04-10 14:54:13
 *  Author: dsosnowski
 */ 

#define F_CPU 12000000UL
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "serial.h"
#include "defs.h"
#include "ui.h"

//0 - ND; 1 - PN; 2 - WT; 3 - SR; 4 - CZ; 5 - PT; 6 - SB
const char namesofday[7][3] = {"Nd\0", "Pn\0", "Wt\0", "Sr\0", "Cz\0", "Pt\0", "Sb\0"};
	
//0...9, 10, -, dot, null
//const char digits[14] = {190, 132, 185, 173, 135, 47, 63, 166, 191, 175, 190, 1, 64, 0};	
const char digits[14] = {63, 5, 155, 143, 165, 174, 190, 39, 191, 175, 63, 128, 64, 0};

#define DOT_VAL			64
#define NULL_VAL		0
#define SELECT_VAL		8
#define MINUS_VAL		128
#define DEGREE_VAL		163
#define CELSIUS_VAL		58

#define MINUS_CHAR		11
#define DOT_CHAR		12
#define NULL_CHAR		13

struct {
	char chars[7];
	char dot;
	char brightness;
} display;

struct {
	char press;
	char level;
	char seconds;
	char minutes;
	char hours;
	char dayofmonth;
	char month;
	char year;
	char dayofweek;
	char leap;
} ui;


#define DISP_REFRESH_FREQ	100
#define DISP_BLANKING_VAL	10
#define DISP_DIMMER_MAX		11


#define UI_SWITCH_THRESHOLD	50
#define UI_SWITCH_REPEAT	2


//--------------------------------------------------------
void UiDispInit(void)
{
	ANODES_PORT = (1<<UART_RXD | 1<<TICK_PIN);
	ANODES_DDR = ~(1<<UART_RXD | 1<<TICK_PIN);

	LED_PORT = 0x00;
	LED_DDR = 0xFF;

	ADCSRA = (1<<ADEN | 1<<ADFR | 1<<ADPS2 | 1<<ADPS1 | 1<<ADPS0);	//turn on, free run, presc = 128
	ADMUX = (1<<REFS0 | 1<<ADLAR | 3);	//ref = vcc, left adj, channel 3
	ADCSRA |= (1<<ADSC);			//start
	
	TCCR1A = 0;
	TCCR1B = (1<<WGM12 | 1<<CS11 | 1<<CS10);		//div 64
	OCR1A = DISP_REFRESH_FREQ;
	TIMSK |= (1<<OCIE1A);
	
	//display.brightness = DISP_DIMMER_MAX;
	display.chars[0] = 0xFF;
	display.chars[1] = 0xFF;
	display.chars[2] = 0xFF;
	display.chars[3] = 0xFF;
	display.chars[4] = 0xFF;
	display.chars[5] = 0xFF;
	sei();
}

//-----------------------------------
//SIGNAL(TIMER1_OVF_vect)
ISR(TIMER1_COMPA_vect)
{
	static char step;
	
	//if (step < display.brightness) 
	if (step < 11)
		step++;
	else 
		step = 0;
	
	if (step & 1)
	{
		//OCR1A = DISP_REFRESH_FREQ;
		OCR1A = (0xFF - display.brightness);	
		switch (step)
		{
			case 1:
			{
				LED_PORT = display.chars[0];
				ANODES_PORT |= (1<<ANODE1_PIN);
			}
			break;
			case 3:
			{
				LED_PORT = display.chars[1];
				ANODES_PORT |= (1<<ANODE2_PIN);
			}
			break;
			case 5:
			{
				LED_PORT = display.chars[2];
				ANODES_PORT |= (1<<ANODE3_PIN);
			}
			break;
			case 7:
			{
				LED_PORT = display.chars[3];
				ANODES_PORT |= (1<<ANODE4_PIN);
			}
			break;
			case 9:
			{
				LED_PORT = display.chars[4];
				ANODES_PORT |= (1<<ANODE5_PIN);
			}
			break;
			case 11:
			{
				LED_PORT = display.chars[5];
				ANODES_PORT |= (1<<ANODE6_PIN);
			}
			break;
		}
	}
	else
	{
		//OCR1A = DISP_BLANKING_VAL;	
		OCR1A = display.brightness;	
		ANODES_PORT = (1<<UART_RXD | 1<<TICK_PIN);
		LED_PORT = NULL_VAL;
	}
}

//-----------------------------------
char *UiDispClockGet(void)
{
	return &ui.seconds;
}

//-----------------------------------
void UiDispDirec(char d1, char d2, char d3, char d4, char d5, char d6)
{
	display.chars[0] = d1;
	display.chars[1] = d2;
	display.chars[2] = d3;
	display.chars[3] = d4;
	display.chars[4] = d5;
	display.chars[5] = d6;
}

//-----------------------------------
void UiDispClock(char *clock)
{
	char d;
	d = (*clock / 10);
	display.chars[4] = (digits[d]);
	d = (*clock % 10);
	display.chars[5] = (digits[d]);

	clock++;
	d = (*clock / 10);
	display.chars[2] = (digits[d]);
	d = (*clock % 10);
	display.chars[3] = (digits[d]);

	clock++;
	d = (*clock / 10);
	if (d)
		display.chars[0] = (digits[d] | DOT_VAL);
	else
		display.chars[0] = (NULL_VAL | DOT_VAL);
	
	d = (*clock % 10);
	display.chars[1] = (digits[d]);		
	
	//display.chars[6] = (digits[DOT_CHAR]);
}

//-----------------------------------
void UiDispDate(char *clock)
{
	char d;
	//clock += 3;
	
	d = (((*clock) + 1) / 10);
	display.chars[0] = (digits[d]);
	d = (((*clock) + 1) % 10);
	display.chars[1] = (digits[d] | DOT_VAL);

	clock++;
	d = (((*clock) + 1) / 10);
	display.chars[2] = (digits[d]);
	d = (((*clock) + 1) % 10);
	display.chars[3] = (digits[d] | DOT_VAL);

	clock++;
	d = (*clock / 10);
	display.chars[4] = (digits[d]);
	d = (*clock % 10);
	display.chars[5] = (digits[d]  | DOT_VAL);
	
	//display.chars[6] = (digits[NULL_CHAR]);
}

//-----------------------------------
void UiDispTemp(int16_t d)
{
	display.chars[0] = NULL_VAL;
	
	if (d >= 1000)	
	{
		display.chars[1] = digits[1];
		d -= 1000;	
	}
	else if (d <= -1)	
	{
		display.chars[1] = MINUS_VAL;
		d *= -1;	
	}
	else 
		display.chars[1] = NULL_VAL;
	
	d /= 10;
	
	display.chars[2] = digits[(d / 10)];
	display.chars[3] = digits[(d % 10)];
	
	display.chars[4] = DEGREE_VAL;
	display.chars[5] = CELSIUS_VAL;
}

//-----------------------------------
void UiDispConfigTimeZone(char *src)
{
	char d;
	
	display.chars[0] = (NULL_VAL  | DOT_VAL);
	d = *src;
	if (d > 14)
	{
		display.chars[0] = (digits[MINUS_CHAR] | DOT_VAL);
		d = (29 - d);		
	}
	if (d >= 10) display.chars[0] |= (digits[1]  | DOT_VAL);
	d = (d % 10);
	display.chars[1] = (digits[d]);

	//src++;
	display.chars[2] = (digits[0]);
	d = (*src % 10);
	display.chars[3] = (digits[0] | DOT_VAL);

	src++;
	display.chars[4] = (NULL_VAL);
	d = (*src % 10);
	display.chars[5] = (digits[d]);
}

//-----------------------------------
void UiDispConfigGps(char *src)
{
	char d;
	
	d = *src;
	if (d > 3) d = 3;
	display.chars[0] = (NULL_VAL);
	display.chars[1] = (digits[d] | DOT_VAL);	

	display.chars[2] = (NULL_VAL);
	display.chars[3] = (NULL_VAL);

}
//-----------------------------------
void UiDispConfigTemp(char *src)
{
	char d;
	d = *src;
	if (d > 1) d = 1;
	display.chars[4] = (NULL_VAL);
	display.chars[5] = (digits[d]);	
}

//--------------------------------------------------------
void UiDispMask(char mask)
{
	if (mask == 1) display.chars[0] = SELECT_VAL;
	if (mask == 1) display.chars[1] = SELECT_VAL;
	if (mask == 2) display.chars[2] = SELECT_VAL;
	if (mask == 2) display.chars[3] = SELECT_VAL;
	if (mask == 3) display.chars[4] = SELECT_VAL;
	if (mask == 3) display.chars[5] = SELECT_VAL;
	if (mask == 4) (display.chars[0] &= ~(DOT_VAL));
}

/*//-----------------------------------
void UiSendAllClock(char *clock)
{
	SerialSimpleVal(*(clock + 2));
	SerialSend(':');
	SerialSimpleVal(*(clock + 1));
	SerialSend(':');
	SerialSimpleVal(*clock);
	SerialSend(' ');

	SerialSimpleVal(*(clock + 3) + 1);
	SerialSend('-');
	SerialSimpleVal(*(clock + 4) + 1);
	SerialSend('-');
	SerialSimpleVal(*(clock + 5));
	SerialSend(' ');
}

//-----------------------------------
void UiSendDay(char *clock)
{
	uint16_t i;
	i = (char)(*(clock + 6));
	SerialSend(' ');
	SerialText(&namesofday[i]);	
	SerialSend(' ');
}

//-----------------------------------
void UiSendLeap(char *clock)
{
	if (*(clock + 7) == 1)
		SerialText(" 1");
	else
		SerialText("  ");
}
*/

//--------------------------------------------------------
void UiKbdInit(void)
{
	SW_DDR &= ~(1<<SW_1);
	SW_DDR &= ~(1<<SW_2);
	SW_PORT |= (1<<SW_1);
	SW_PORT |= (1<<SW_2);
}

//----------------------------------------------
char UiKbdScan(void)
{
	char pin;
	char key;
	static char prev;
	static char hold;
	
	pin = SW_PIN;
	key = 0;
	
	//any state
	if ((pin & (1<<SW_1 | 1<<SW_2)) == (1<<SW_1 | 1<<SW_2))
	{
		//sw_free
		if ((hold > 1) && (hold < UI_SWITCH_THRESHOLD)) key = prev;
		prev = 0;
		hold = 0;
	}
	else
	{
		//if pressed
		if ((pin & (1<<SW_1)) == 0)	key = UI_SWITCH1;
		if ((pin & (1<<SW_2)) == 0)	key = UI_SWITCH2;

		if (hold < 200) hold++;			//counting hold time
		if (key != prev) hold = 0;		//new key - reset counters
		prev = key;
		key = 0;

		if (hold == (UI_SWITCH_THRESHOLD + UI_SWITCH_REPEAT)) //after threshold - repeat key
		{
			key =  (prev + UI_SWITCH_HOLD);
			//if (prev == UI_SWITCH2)
			//hold = UI_SWITCH_THRESHOLD;
		}
	}
	
	ui.press = key;
	return key;
}

//----------------------------------------------
char UiKbdGet(char clear)
{
	if (clear) ui.press = 0;
	return ui.press;
}

//----------------------------------------------
char UiKbdState(void)
{
	if ((SW_PIN & (1<<SW_1)) == 0)	return UI_SWITCH1;
	if ((SW_PIN & (1<<SW_2)) == 0)	return UI_SWITCH2;
}


//--------------------------------------------------------
void UIBrightness(void)
{
	static char cycle;
	static uint16_t temp;
	
	temp += ADCH;
	cycle++;
	
	if (cycle >= 5)
	{
		temp /= cycle;
		if (temp > 128)
			temp -= 128;
		else
			temp = 0;
			
		temp *= 2;
		if (temp > 245) temp = 245;
		
		if (display.brightness < temp) display.brightness++;
		if (display.brightness < temp) display.brightness++;
		if (display.brightness > temp) display.brightness--;
		if (display.brightness > temp) display.brightness--;
		
		temp = 0;
		cycle = 0;
	}
}	